#!/bin/ksh
# Licensed Materials - Property of IBM
# Restricted Materials of IBM

# Copyright IBM Corp. 2006 All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
# Release Name: omdomino_dev
#
# Description: Start up the Domino/Intelliwatch CPC(kgbclient).
#              Restart the CPC when the Domino Server has
#              shutdown.
#
# Parameters:
#   CANDLEHOME, KGB_NOTESINIPATH, and KGB_INSTANCE_NAME
#   set in the environment
#
# Returns:
#    None
#======================================================
#

process_running()
{
# takes process name to search for
set -x

if [[ "$binArch" = sol+([0-9]) ]]
then
  srun=`ps -u $1 -o "args" | grep -v "grep" | grep "$2"`
else # aix, linux, etc
  srun=`ps -u $1 -o "%a" | grep -v "grep" | grep "$2"`
fi
echo "`date` : User: $1, Process: $2, List=$srun" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
if [ -n "$srun" ]; then
  return 0
else
  return 1
fi
}

# turn on tracing for log file
set -x

# save the parent pid
KGBAGENT_PID=$PPID

# make sure CANDLEHOME exists
if [ ! -d "$CANDLEHOME" ]; then
  echo "ERROR: CANDLEHOME was not found, exiting..."
  exit
fi

# make sure the instance name exists
if [ -z "$KGB_INSTANCE_NAME" ]; then
  echo "ERROR: KGB_INSTANCE_NAME was not found, exiting..."
  exit
fi

# truncate log file and modify permissions
cat /dev/null > $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
chmod 666 $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
echo "`date` : kgbstart.sh called for instance $KGB_INSTANCE_NAME" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log

# make sure the notes.ini path exists
KGB_NOTESINIPATH=`echo $KGB_NOTESINIPATH | sed -e "s/notes.ini//g"`
if [ ! -d "$KGB_NOTESINIPATH" ]; then
  echo "`date` : KGB_NOTESINIPATH : $KGB_NOTESINIPATH/notes.ini was not found, exiting..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  exit
fi

# setup path to the machine architecture
# source dynarch.shl in a subshell so it does not modify the environment
binArch=`( . $CANDLEHOME/bin/dynarch.shl $CANDLEHOME/registry/archdsc.tbl; setBinLibArchs $cArch gb; echo $binArch )`

# export as KGB_BINARCH as BSS1_genenv gets confused on binArch
KGB_BINARCH=$binArch
export KGB_BINARCH

# set for APAR #IZ36147
chmod 777 $CANDLEHOME/$binArch/gb/bin/kgbstart.sh

# determine the Lotus Domino base directory
LOTUSBASEDIR=`cat $KGB_NOTESINIPATH/notes.ini | grep -i NotesProgram | awk -F= {'print $2'}`
while true
do
  if [ -f $LOTUSBASEDIR/bin/server ]; then
    break
  else
    LOTUSBASEDIR=`dirname $LOTUSBASEDIR`
    if [ "$LOTUSBASEDIR" = "/" -o "$LOTUSBASEDIR" = "." ]; then
      break
    fi
  fi
done
if [ ! -f $LOTUSBASEDIR/bin/server ]; then
  echo "`date` : Using the NotesProgram keyword in $KGB_NOTESINIPATH/notes.ini to find the Lotus base directory, ${LOTUSBASEDIR} directory was not found, exiting..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  exit
fi

# check the Lotus Domino user id and group id was used to start the agent
LOTUSGROUP=`ls -l ${KGB_NOTESINIPATH}/notes.ini | awk '{print $4}'`
LOTUSOWNER=`ls -l ${KGB_NOTESINIPATH}/notes.ini | awk '{print $3}'`
if [ -z "$LOTUSOWNER" ]; then
  echo "`date` : Could not determine the Domino user id, exiting..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  exit
fi
if [[ "$binArch" = sol+([0-9]) ]]
then
  /usr/xpg4/bin/id -un | grep "^$LOTUSOWNER" >/dev/null 2>&1
else
  id -un | grep "^$LOTUSOWNER" >/dev/null 2>&1
fi
if [ $? -ne 0 ]; then
  ROOTUSER=yes
fi

# APAR 36147
#chown $LOTUSOWNER:$LOTUSGROUP $CANDLEHOME/$binArch/gb/bin/KGBCLIENTBRIDGE.lock

# setup environement based on architecture
if [[ "$binArch" = aix+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/ibmpow:$LOTUSBASEDIR/notes/latest/ibmpow/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/ibmpow
  export Notes_ExecDirectory
  LIBPATH=$LIBPATH:$LOTUSBASEDIR/notes/latest/ibmpow
  export LIBPATH
fi
if [[ "$binArch" = sol+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/sunspa:$LOTUSBASEDIR/notes/latest/sunspa/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/sunspa
  export Notes_ExecDirectory
  LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LOTUSBASEDIR/notes/latest/sunspa
  export LD_LIBRARY_PATH
fi
if [[ "$binArch" = li+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/linux:$LOTUSBASEDIR/notes/latest/linux/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/linux
  export Notes_ExecDirectory
  LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LOTUSBASEDIR/notes/latest/linux
  export LD_LIBRARY_PATH
fi
if [[ "$binArch" = ls+([0-9]) ]]
then
  PATH=$PATH:$LOTUSBASEDIR/bin:$LOTUSBASEDIR/notes/latest/zlinux:$LOTUSBASEDIR/notes/latest/zlinux/res/C
  export PATH
  Notes_ExecDirectory=$LOTUSBASEDIR/notes/latest/zlinux
  export Notes_ExecDirectory
  LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LOTUSBASEDIR/notes/latest/zlinux
  export LD_LIBRARY_PATH
fi

# chk for the required softlinks if root
if [ -n "$ROOTUSER" ]; then
  # chk to make sure the agent has been configured
  if [ ! -f $LOTUSBASEDIR/bin/kgbclient -o ! -f $Notes_ExecDirectory/kgbclient ]; then
    echo "`date` : Adding softlinks for $KGB_INSTANCE_NAME" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    $CANDLEHOME/$binArch/gb/bin/kgbconfig.sh $KGB_INSTANCE_NAME kgbstart >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  fi
fi

# remove the status file
if [ -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out ]; then
  echo "`date` : removing $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  rm -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
fi

# chk for Domino Sever shutdown(1) and restart the client
# when the Domino server process ends
CNT=1
MAXCNT=150
while true
do
  # start the Domino agent, switch to the notes user if root
  cd "$KGB_NOTESINIPATH"
  if [ -z "$ROOTUSER" ]; then
    echo "`date` : starting $LOTUSBASEDIR/bin/kgbclient as the $LOTUSOWNER user" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    $LOTUSBASEDIR/bin/kgbclient $KGB_INSTANCE_NAME
    CR=$?
  else
    echo "`date` : Switching to the $LOTUSOWNER user and starting $LOTUSBASEDIR/bin/kgbclient" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    su $LOTUSOWNER -c "$LOTUSBASEDIR/bin/kgbclient $KGB_INSTANCE_NAME"
    CR=$?
  fi
  echo "`date` : kgbclient exited with return code(${CR})" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  # if return code is 31, server is shutting down
  if [ $CR -eq 31 ]; then
    # set the status of server and cluster to recycling
    echo 4 > $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out 
    echo 5 >> $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
    echo "`date` : Starting 5 minute timeout for server process: $Notes_ExecDirectory/server" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
    while true
    do
      # chk if server process is running
      # The $Notes_ExecDirectory could be a symbolic link, not the actual path
      # Change from search $LOTUSOWNER $Notes_ExecDirectory/server to 
      # $LOTUSOWNER server
      process_running $LOTUSOWNER server
      if [ $? -eq 1 ]; then
        break
      fi
      echo "`date` : timeout count=$CNT maxcount=$MAXCNT" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
      CNT=`expr $CNT + 1`
      echo "`date` : sleeping" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
      sleep 2
      if [ $CNT -gt $MAXCNT ]; then
        echo "`date` : timeout waiting for $Notes_ExecDirectory/server process to stop, exiting..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        echo "`date` : killing the kgbagent process($KGBAGENT_PID) for $KGB_INSTANCE_NAME instance..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
        kill -15 $KGBAGENT_PID
        if [ -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out ]; then
          echo "`date` : removing $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
          rm -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
        fi
        exit
      fi
    done
    CNT=1
    # sleep some more to make sure
    sleep 50
    echo "`date` : restarting kgbclient" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  else
    if [ $CR -ne 30 ]; then
      # kill the kgbagent as kgbclient went down unexpectly
      echo "`date` : killing the kgbagent process($KGBAGENT_PID) for $KGB_INSTANCE_NAME instance..." >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
      kill -15 $KGBAGENT_PID
    fi
    # break and exit
    break
  fi
done

# remove the status file
if [ -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out ]; then
  echo "`date` : removing $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
  rm -f $CANDLEHOME/$binArch/gb/bin/kgb_${KGB_INSTANCE_NAME}_status.out
fi

echo "`date` : exiting" >> $CANDLEHOME/logs/kgbstart_$KGB_INSTANCE_NAME.log
